@@ -6,13 +6,13 @@ use Class::AutoClass;
 use Text::Abbrev;
 @ISA = qw(Class::AutoClass); # AutoClass must be first!!
 
-##BEGIN {
+
   @AUTO_ATTRIBUTES=qw(name 
 		      _keys);
   @OTHER_ATTRIBUTES=qw(keys);
   %SYNONYMS=();
   Class::AutoClass::declare(__PACKAGE__);
-##}
+
 sub _init_self {
   my($self,$class,$args)=@_;
   return unless $class eq __PACKAGE__; # to prevent subclasses from re-running this
@@ -27,8 +27,9 @@ my %CODES=abbrev @CODES;
 my %TYPES=(string  =>'longtext',
 	   integer =>'int',
 	   float   =>'double',
-	   object  =>'int',);
-my @TYPES=keys %TYPES;
+	   object  =>'int',
+	   mixed   => 'longtext',);
+my @TYPES=CORE::keys %TYPES;
 my %TYPES_ABBREV=abbrev @TYPES;
 
 sub schema {
